@@ -35,8 +35,6 @@ logger = getLogger(__name__)
 PATH_TO_GLADIA_SRC = os.getenv("PATH_TO_GLADIA_SRC", "/app")
 ENV_YAML = "env.yaml"
 
-MODELS_FOLDER_SUFFIX = "models"
-
 FILE_TYPES = ["image", "audio", "video"]
 TEXT_TYPES = ["text", "str", "string"]
 LIST_TYPES = ["list", "tuple", "set"]
@@ -66,40 +64,6 @@ def merge_dicts(*args: dict) -> dict:
     return dict(sum_items)
 
 
-def to_task_name(folder_name: str) -> str:
-    """
-    Convert a folder name to a task name.
-
-    Args:
-        folder_name (str): name of the folder
-
-    Returns:
-        str: name of the task
-    """
-
-    # remove the models suffix
-    # remove 1 more character to remove the "-"
-    return folder_name[: -(len(MODELS_FOLDER_SUFFIX) + 1)]
-
-
-def to_models_folder_name(model_name: str) -> str:
-    """
-    This function is used to find the folder containing all
-    related models for a given task.
-    We use the -{MODELS_FOLDER_SUFFIX} in order to
-    avoid fastapi to be confused with the routing layer
-    defined by the task.py
-
-    Args:
-        model_name (str): name of the model
-
-    Returns:
-        str: name of the folder containing all related models
-
-    """
-    return f"{model_name}-{MODELS_FOLDER_SUFFIX}"
-
-
 def dict_model(name: str, dict_def: dict) -> BaseModel:
     """
     Create a model from a dictionary.
@@ -143,7 +107,7 @@ def get_module_infos(root_path=None) -> Tuple:
     pwd = str(Path(caller_file).absolute()).split("/")
 
     plugin = pwd[len(pwd) - 3 : len(pwd)]
-    tags = ".".join(plugin)[:-3]
+    tags = ".".join(plugin)
     task = plugin[-1][:-3]
 
     return task, plugin, tags
@@ -152,8 +116,6 @@ def get_module_infos(root_path=None) -> Tuple:
 def get_model_versions(root_path: str = None) -> Tuple[List[str], str]:
     """
     Get the list of available model versions.
-    We use the -{MODELS_FOLDER_SUFFIX} in order to
-    avoid fastapi to be confused with the routing layer
 
     Args:
         root_path (str): path to the root of the project
@@ -174,7 +136,6 @@ def get_model_versions(root_path: str = None) -> Tuple[List[str], str]:
         rel_path = os.path.join(cwd, rel_path)
 
     sub_dir_to_crawl = f"{os.path.splitext(os.path.basename(rel_path))[0]}"
-    sub_dir_to_crawl = to_models_folder_name(sub_dir_to_crawl)
 
     versions = dict()
     package_path = str(Path(rel_path).parent.joinpath(sub_dir_to_crawl))
@@ -187,9 +148,7 @@ def get_model_versions(root_path: str = None) -> Tuple[List[str], str]:
             # Retieve metadata from metadata file and push it to versions,
             # the output of the get road
             model = fname
-            endpoint = package_path.replace("apis", "").replace(
-                f"-{MODELS_FOLDER_SUFFIX}", ""
-            )
+            endpoint = package_path.replace("apis", "")
             model_metadata = get_model_metadata(endpoint, model)
             versions[fname] = model_metadata
 
@@ -208,7 +167,7 @@ def get_model_metadata(endpoint: str, model: str) -> dict:
         dict: metadata of the model
     """
     splited_endpoint = endpoint.split("/")
-    endpoint = f"/{splited_endpoint[1]}/{splited_endpoint[2]}/{splited_endpoint[3]}-{MODELS_FOLDER_SUFFIX}/"
+    endpoint = f"/{splited_endpoint[1]}/{splited_endpoint[2]}/{splited_endpoint[3]}/"
     path = f"apis{endpoint}{model}"
     file_name = ".model_metadata.yaml"
     fallback_file_name = ".metadata_model_template.yaml"
@@ -227,7 +186,7 @@ def get_task_metadata(endpoint) -> dict:
         dict: metadata of the task
     """
     path = f"apis{endpoint}"
-    file_name = ".task_metadata.yaml"
+    file_name = "task.yaml"
     fallback_file_name = ".metadata_task_template.yaml"
 
     return get_metadata(path, file_name, fallback_file_name)
@@ -331,6 +290,11 @@ def get_module_env_name(module_path: str) -> Union[str, None]:
     """
 
     if os.path.isfile(os.path.join(module_path, ENV_YAML)):
+        task_custom_env = yaml.safe_load(open(os.path.join(module_path, ENV_YAML)))
+
+        if "name" in task_custom_env:
+            return task_custom_env["name"]
+
         path = os.path.join(module_path, ENV_YAML).split("/")
 
         task = path[-3]
@@ -339,6 +303,14 @@ def get_module_env_name(module_path: str) -> Union[str, None]:
         return f"{task}-{model}"
 
     elif os.path.isfile(os.path.join(module_path, "../", ENV_YAML)):
+
+        model_custom_env = yaml.safe_load(
+            open(os.path.join(module_path, "../", ENV_YAML))
+        )
+
+        if "name" in model_custom_env:
+            return model_custom_env["name"]
+
         return os.path.split(os.path.split(os.path.split(module_path)[0])[0])[1]
 
     else:
@@ -459,7 +431,12 @@ class TaskRouter:
     """
 
     def __init__(
-        self, router: APIRouter, input: List[dict], output, default_model: str
+        self,
+        router: APIRouter,
+        input: List[dict],
+        output,
+        default_model: str,
+        rel_path=None,
     ):
         """
         Initialize the TaskRouter class
@@ -479,13 +456,16 @@ class TaskRouter:
         self.output = output
         self.default_model = default_model
 
-        namespace = sys._getframe(1).f_globals
+        self.models_env = {}
 
-        # concate the package name (i.e apis.text.text) with the model filename (i.e word-alignment.py) to obtain the relative path
-        rel_path = os.path.join(
-            namespace["__package__"].replace(".", "/"),
-            namespace["__file__"].split("/")[-1],
-        )
+        if not rel_path:
+            namespace = sys._getframe(1).f_globals
+
+            # concate the package name (i.e apis.text.text) with the model filename (i.e word-alignment.py) to obtain the relative path
+            rel_path = os.path.join(
+                namespace["__package__"].replace(".", "/"),
+                namespace["__file__"].split("/")[-1],
+            )
 
         full_path = os.path.join(os.getenv("PATH_TO_GLADIA_SRC", "/app"), rel_path)
 
@@ -559,6 +539,8 @@ class TaskRouter:
             default=Query(self.default_model, enum=set(self.versions.keys())),
         )
 
+        self.inputs = self.__get_routeur_inputs()
+
         # Define the post routes implemented by fastapi
         # The @router.post() content define the informations
         # displayed in /docs and /openapi.json for the post routes
@@ -574,14 +556,10 @@ class TaskRouter:
 
             # cast BaseModel pydantic models into python type
             parameters_in_body = self.__build_parameters_in_body(kwargs)
+            del kwargs
 
-            kwargs = parameters_in_body
-
-            self.__get_routeur_inputs()
-
-            model = kwargs["model"]
-            # remove it from kwargs to avoid passing it to the predict function
-            del kwargs["model"]
+            model = parameters_in_body["model"]
+            del parameters_in_body["model"]
 
             module_path = f"{self.root_package_path}/{model}/"
             self.module_path = module_path
@@ -593,15 +571,21 @@ class TaskRouter:
 
             # return False if an error occured
             (
-                kwargs,
+                parameters_in_body,
                 success,
                 error_message,
-            ) = await clean_kwargs_based_on_router_inputs(kwargs, self.inputs)
+            ) = await clean_kwargs_based_on_router_inputs(
+                parameters_in_body, self.inputs
+            )
 
             if not success:
                 raise HTTPException(status.HTTP_400_BAD_REQUEST, error_message)
 
-            env_name = get_module_env_name(module_path)
+            if module_path not in self.models_env:
+                self.models_env[module_path] = get_module_env_name(module_path)
+
+            env_name = self.models_env[module_path]
+
             # if its a subprocess
             if env_name is not None:
 
@@ -614,7 +598,7 @@ class TaskRouter:
                     # if the api is running, use it
                     result = call_subprocess_api(
                         api_name=api_name,
-                        kwargs=kwargs,
+                        kwargs=parameters_in_body,
                     )
 
                 else:
@@ -622,27 +606,23 @@ class TaskRouter:
                     result = self.__call_in_subprocess(
                         inputs=self.inputs,
                         model=model,
-                        kwargs=kwargs,
+                        kwargs=parameters_in_body,
                         env_name=env_name,
                     )
 
             else:
                 result = self.__call_in_a_native_way(
-                    model=model, kwargs=kwargs, args=args
+                    model=model, kwargs=parameters_in_body, args=args
                 )
 
             return self.__post_processing(result)
 
     def __get_routeur_inputs(self) -> list:
-        self.routeur = (
-            to_task_name(self.root_package_path)
-            .replace(os.getenv("PATH_TO_GLADIA_SRC", "/app"), "")
-            .replace("/", ".")
-            .strip(".")
+        task_metadata = yaml.safe_load(
+            open(os.path.join(self.root_package_path, "task.yaml"))
         )
-        self.this_routeur = importlib.import_module(self.routeur.replace("/", "."))
-        self.inputs = self.this_routeur.inputs
-        return self.inputs
+
+        return task_metadata["inputs"]
 
     def __build_parameters_in_body(self, kwargs: dict) -> dict:
         """
@@ -804,14 +784,14 @@ class TaskRouter:
         # convert io Bytes to files
         # input_files to clean
         input_files = list()
-        for input in inputs:
-            if input["type"] in FILE_TYPES:
-                tmp_file = write_tmp_file(kwargs[input["name"]])
-                kwargs[input["name"]] = tmp_file
+        for input_name, input_metadata in inputs.items():
+            if input_metadata["type"] in FILE_TYPES:
+                tmp_file = write_tmp_file(kwargs[input_name])
+                kwargs[input_name] = tmp_file
                 input_files.append(tmp_file)
 
-            elif input["type"] in ["text"]:
-                kwargs[input["name"]] = quote(kwargs[input["name"]])
+            elif input_metadata["type"] in ["text"]:
+                kwargs[input_name] = quote(kwargs[input_name])
 
         output_tmp_result = quote(tempfile.NamedTemporaryFile().name)
 
@@ -900,10 +880,8 @@ async def clean_kwargs_based_on_router_inputs(
     success = True
     error_message = "Empty error message"
 
-    for input in inputs:
-        input_name = input["name"]
-
-        if input["type"] in FILE_TYPES:
+    for input_name, input_metadata in inputs.items():
+        if input_metadata["type"] in FILE_TYPES:
             # if the input file is in kwargs:
             if isinstance(
                 kwargs.get(input_name, None),
@@ -939,13 +917,11 @@ async def clean_kwargs_based_on_router_inputs(
             # remove the url arg to avoid it to be passed in predict
             if f"{input_name}_url" in kwargs:
                 del kwargs[f"{input_name}_url"]
-        elif input["type"] in LIST_TYPES:
+        elif input_metadata["type"] in LIST_TYPES:
             kwargs[input_name] = str(kwargs[input_name].value)
         else:
             if not kwargs.get(input_name, None):
-                error_message = (
-                    f"Input '{input_name}' of '{input['type']}' type is missing."
-                )
+                error_message = f"Input '{input_name}' of '{input_metadata['type']}' type is missing."
                 success = False
 
     return kwargs, success, error_message
